/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <gnome.h>
#include <sys/msg.h>
#include "Communication.h"
#include "Commun.h"
#include "../Debug.h"
#include "Initialisation_Com.h"
#include "../Equipement.h"
#include "../Equipement_Port.h"
#include "../Script.h"
#include "../Interface/Root_Interface.h"
#include "../Nessus.h"


struct Struct_Modele_Communication Liste_Communication[] = {
	{AGENT_VERSION, &Reception_Version_Agent},
	{AJOUT_PORT, NULL},
	{AJOUT_SIGNATURE, NULL},
	{RESTAURATION_MACHINE, NULL},
	{AJOUT_RESEAU_PRIVEE, NULL},
	{DEMANDE_LISTE_INTERFACE, NULL},
	{NOUVELLE_INTERFACE, &Recuperation_Nouvelle_Interface},
	{FIN_LISTE_INTERFACE, &Recuperation_Liste_Interface_Fini},
	{DEMARRAGE_AGENT, NULL},
	{DEMANDE_ARRET_AGENT, NULL},
	{RETOUR_MESSAGE, &Recuperation_Message_Agent},
	{RETOUR_ERREUR_CRITIQUE, NULL},
	{INFO_AGENT, &Retour_Info},
	{NOUVEAU_EQUIPEMENT, &Recuperation_Nouveau_Equipement},
	{DEMANDE_NESSUS, NULL},
	{DEMARRAGE_NESSUS, &Demarrage_Nessus},
	{RETOUR_NESSUS_INFO, &Nouvelle_Info_Nessus},                  
	{RETOUR_NESSUS_DONNEE, &Nouvelle_Donnee_Nessus},
	{RETOUR_NESSUS_FIN, &Fin_Nessus},
	{DEMANDE_RAFRAICHIR_ETAT_EQUIMEMENT, NULL},
	{RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT, NULL},
	{DEMANDE_RAFRAICHIR_LISTE_PORTS, NULL},
	{RETOUR_RAFRAICHIR_LISTE_PORTS, NULL},
	{DEMANDE_RAFRAICHIR_ENREGISTREMENT_SAMBA, NULL},
	{RETOUR_RAFRAICHIR_ENREGISTREMENT_SAMBA, NULL},
	{DEMANDE_RAFRAICHIR_SIGNATURE, NULL},
	{RETOUR_RAFRAICHIR_SIGNATURE, NULL},
	{DEMANDE_NOTIFICATION_WARNING_ETAT_EQUIPEMENT, NULL},
	{DEMANDE_NOTIFICATION_WARNING_ETAT_PORT, NULL},
	{DEMANDE_NOTIFICATION_CRITIQUE_ETAT_EQUIPEMENT, NULL},
	{DEMANDE_NOTIFICATION_CRITIQUE_ETAT_PORT, NULL},
	{RETOUR_NOTIFICATION_CHANGEMENT_ETAT_EQUIPEMENT, &Retour_Notification_Equipement},
	{RETOUR_NOTIFICATION_CHANGEMENT_ETAT_PORT, &Retour_Notification_Port},
	{ANNULATION_NOTIFICATION_ETAT_EQUIPEMENT, NULL},
	{ANNULATION_NOTIFICATION_ETAT_PORT, NULL},
	{ACTIVE_ALERTE_INTRUS, NULL},
	{DESACTIVE_ALERTE_INTRUS, NULL},
	{ALERTE_INTRUS, &Retour_Alerte_Intrusion},
	{DEMANDE_MODIFIE_NIVEAU_DEBUG, NULL},
	{RETOUR_SCRIPT, &Retour_Script},
	{RETOUR_SCRIPT_TERMINER, &Retour_Script_Terminer}
};


void Recuperation_Nb_Octet_Recu (gint Nb_Octet, struct Struct_Liste_Reseau *Reseau)
{
	if (Reseau != NULL)
		Reseau->Octet_Recu_Agent = Reseau->Octet_Recu_Agent + Nb_Octet;
}

gboolean Rachaichi_Info_Transfert_Entre_Agent ()
{
	gchar    *Text;

	if (Reseau_Selectionner != NULL) {
		Text = g_strdup_printf ("R : %.1f Ko", (((double) Reseau_Selectionner->Octet_Recu_Agent) / 1024));
		gtk_label_set_text (GTK_LABEL (label_Inter_AutoScan_Reception), Text);
		g_free (Text);
		Text = g_strdup_printf ("S : %.1f Ko", (((double) Reseau_Selectionner->Octet_Envoyer_Agent) / 1024));
		gtk_label_set_text (GTK_LABEL (label_Inter_AutoScan_Emission), Text);
		g_free (Text);
		Reseau_Selectionner->Octet_Recu_Agent = 0;
		Reseau_Selectionner->Octet_Envoyer_Agent = 0;
	}
	return TRUE;
}


gboolean Communication_Reception_Donnee_Boucle ()
{
	struct struct_Info Donnee;
	gint      Taille_Donnee_Recue;

	do {
		bzero (&Donnee, sizeof (struct struct_Info));
		if ((Taille_Donnee_Recue = msgrcv (File_Info, (void *) &Donnee, sizeof (struct struct_Info) - sizeof (long), Mon_Pid, IPC_NOWAIT)) > -1) {
			if (Liste_Communication[Donnee.ID_Info].Gestion_Communication != NULL)
				(*Liste_Communication[Donnee.ID_Info].Gestion_Communication) ((gpointer *) & Donnee.Donnee);
			else
				debug_printf (INTERFACE, D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Reception Message sans action defini");
		}
	}
	while (Taille_Donnee_Recue != -1);
	return TRUE;
}


void Communication_Initialisation ()
{
	Mon_Pid = getpid ();
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "AutoScan : pid(%d)", Mon_Pid);
	
	if (msgctl (msgget (KEY_FILE, 0), IPC_RMID, NULL) == 0)
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "File KEY_FILE detruite");
	
	if ((File_Info = msgget (KEY_FILE, IPC_CREAT | 0777)) == -1)
		debug_printf (INTERFACE, D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "msgget  : %s", strerror (errno));
}
