/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <gnome.h>
#include "Commun.h"
#include "Initialisation_Com.h"
#include "../Declaration_Constante.h"
#include "../Debug.h"
#include "../Settings_Port.h"
#include "../Settings.h"
#include "../Settings_Signature.h"
#include "../Interface/Add_Network_Interface.h"
#include "../Add_Network_Subnet.h"
#include "../Interface/Combo.h"
#include "../Interface/Add_Network_Callbacks.h"
#include "../Gconf.h"
#include "../Restoration_Agent.h"
#include "Communication.h"

void Efface_Toute_Interface_Combo ()
{
	Efface_Combo (combobox_Select_Interface);
}


void Demande_Arret_Agent (struct Struct_Liste_Reseau *Reseau)
{
	struct struct_Demande_Arret_Agent Message;
		
	bzero (&Message, sizeof (struct struct_Demande_Arret_Agent));
	Message.ID_Reseau = Reseau->Id_Reseau;
	Communication_Envoie_Donnee (Reseau->Pid_Agent, DEMANDE_ARRET_AGENT, (gpointer *) & Message, sizeof (Message), NULL);
}


void Demande_Demarrage_Agent (gint ID_Reseau, gboolean Autoriser_A_Scanner)
{
	struct struct_Demarrage_Agent Message;
	gchar    *combo = NULL, *pCombo = NULL;

	bzero (&Message, sizeof (struct struct_Demarrage_Agent));
	Message.ID_Reseau = ID_Reseau;
	combo = Recuperation_Texte_Combo (combobox_Select_Interface);
	if ((pCombo = index (combo, ':')) == NULL)
		debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Chaine Interface");
	memcpy (&Message.Interface, combo, pCombo - combo - 1);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Interface : [%s]\n", Message.Interface);
	Message.Nb_Thread_Max = Lecture_Gconf_Int (INT_THREAD_MAX);
	Message.Autoriser_A_Scanner = Autoriser_A_Scanner;
	Message.Niveau_Debugage = Debug; 
	Communication_Envoie_Donnee (Pid_Agent_Initialisation, DEMARRAGE_AGENT, (gpointer *) & Message, sizeof (Message), NULL);
	g_free (combo);		
}

void Recuperation_Message_Agent (gpointer * Donnee)
{
	struct struct_Retour_Message *Message = NULL;

	ERROR_if (Donnee == NULL);
	Message = (struct struct_Retour_Message *) Donnee;
	debug_printf (Message->ID_Reseau, Message->Type_message, Message->File, Message->Fonction, Message->Line, Message->Message);
	Recuperation_Nb_Octet_Recu (sizeof (struct struct_Retour_Message), Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau));
}


void Recuperation_Nouvelle_Interface (gpointer * Donnee)
{
	struct struct_Nouvelle_Interface *Interface;

	ERROR_if (Donnee == NULL);
	Interface = (struct struct_Nouvelle_Interface *) Donnee;
	Ajoute_Interface_Combo (Interface->Interface);
}

void Recuperation_Liste_Interface_Fini (gpointer * Donnee)
{
	ERROR_if (Donnee == NULL);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Initialize), 1);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Initialisation : completed "));
	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_Select_Interface), 0);	
	Agent_Initialiser = TRUE;	
}

void Demande_Liste_Interface ()
{
	struct struct_Demande_Liste_Interface Message;

	bzero (&Message, sizeof (struct struct_Demande_Liste_Interface));
	Communication_Envoie_Donnee (Pid_Agent_Initialisation, DEMANDE_LISTE_INTERFACE, (gpointer *) & Message, sizeof (Message), NULL);
}


void Initialisation_Agent_Ajout_Sous_Reseau_Autorise ()	
{
	struct Struct_Subnet *Subnet = NULL;
	struct struct_Ajout_Reseau_Privee Message;
	guint     Nb_Objet_Liste, i;

	bzero (&Message, sizeof (struct struct_Ajout_Reseau_Privee));
	Nb_Objet_Liste = g_list_length (GList_Subnet);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Envoie a l'agent la liste des subnet (%d)", Nb_Objet_Liste);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Subnet = g_list_nth_data (GList_Subnet, i);
		Message.Masque = Subnet->Masque;
		Message.A = Subnet->A;
		Message.B = Subnet->B;
		Message.C = Subnet->C;
		Communication_Envoie_Donnee (Pid_Agent_Initialisation, AJOUT_RESEAU_PRIVEE, (gpointer *) & Message, sizeof (Message), NULL);
	}
}


void Initialisation_Agent_Ajout_Port ()	
{
	struct Struct_Liste_Port *Port = NULL;
	struct struct_Ajout_Port Message;
	guint     Nb_Objet_Liste, i;

	bzero (&Message, sizeof (struct struct_Ajout_Port));
	Nb_Objet_Liste = g_list_length (Settings->GList_Port);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Envoie a l'agent la liste des ports (%d)", Nb_Objet_Liste);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port = g_list_nth_data (Settings->GList_Port, i);
		if (Port->Port_Pere == NULL) {	
			Message.Num_Port = Port->Num_Port;
			Message.Serveur = (Port->Type == SERVEUR);
			Communication_Envoie_Donnee (Pid_Agent_Initialisation, AJOUT_PORT, (gpointer *) & Message, sizeof (Message), NULL);
		}
	}
}


void Reception_Version_Agent (gpointer * Donnee)
{
	struct struct_Agent_Version *Agent_Version = NULL;
	struct struct_Demande_Arret_Agent Message;

	ERROR_if (Donnee == NULL);
	Agent_Version = (struct struct_Agent_Version *) Donnee;
	Pid_Agent_Initialisation = Agent_Version->pid;
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Pid de l'agent : (%d)", Pid_Agent_Initialisation);
	if (strcmp (Agent_Version->Version, VERSION_AGENT) == 0) {
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Version AutoScan_Agent : OK");
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Ports list"));
		Initialisation_Agent_Ajout_Port ();
		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Initialize), 0.4);
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Interface List"));
		Demande_Liste_Interface ();
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_Network_Restore))) {
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Initialize), 0.6);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Restore equipment"));
			Restoration_Agent ((gchar *) gtk_entry_get_text (GTK_ENTRY (combo_entry_Network_Restore)));
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Initialize), 0.6);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Restore Subnet list"));
			
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Initialize), 0.8);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Subnet list"));
			Initialisation_Agent_Ajout_Sous_Reseau_Autorise ();
		}
		else {
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Initialize), 0.8);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Subnet list"));
			Initialisation_Agent_Ajout_Sous_Reseau_Autorise ();
		}
	}
	else {
		debug_printf (INTERFACE, D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Version check failed");
		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Initialize), 1);
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Initialize), _("Version check failed"));
		bzero (&Message, sizeof (struct struct_Demande_Arret_Agent));
		Message.ID_Reseau = ID_RESEAU_PAS_INITIALISE;
		Communication_Envoie_Donnee (Pid_Agent_Initialisation, DEMANDE_ARRET_AGENT, (gpointer *) & Message, sizeof (Message), NULL);
	}
}
