/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "Equipement.h"
#include "Equipement_Port.h"
#include "Com/Commun.h"
#include "Com/Communication.h"
#include "Debug.h"
#include "Objet.h"
#include "Interface/Alert_Interface.h"
#include "Interface/Root_Interface.h"
#include "Equipement_Historique.h"
#include "Equipement_Signature.h"
#include <time.h>


void Retour_Info (gpointer * Donnee)
{
	struct struct_Info_Agent *Message = (struct struct_Info_Agent *) Donnee;
	struct Struct_Liste_Reseau *Reseau = NULL;

	ERROR_if (Donnee == NULL);
	Reseau = Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau);
	Reseau->Nb_Equipement = Message->Nb_Machine_Dans_La_Liste;
	Reseau->Nb_Equipement_En_Cours_De_Scan = Message->Nb_Machine_En_Cours_De_Scan;
	Reseau->Nb_Equipement_En_Attente_De_Scan = Message->Nb_Machine_En_Attente_De_Scan;
	Reseau->Octet_Recu = Message->Octet_Recu;
	Reseau->Octet_Envoyer = Message->Octet_Envoyer;
	Affiche_Info_Reseau (Reseau);
	Recuperation_Nb_Octet_Recu (sizeof (struct struct_Info_Agent), Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau));
}


void Retour_Alerte_Intrusion (gpointer * Donnee)
{
	struct struct_Alerte_Intrus *Message = (struct struct_Alerte_Intrus *) Donnee;	
	struct Struct_Liste_Reseau *Reseau = NULL;
	gchar    *Texte_Alerte = NULL;

	ERROR_if (Donnee == NULL);
	Reseau = Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau);
	Texte_Alerte = g_strdup_printf ("Intrusion Alert: \nIP : %d.%d.%d.%d\nAdresse Mac : %02X:%02X:%02X:%02X:%02X:%02X\nNetwork : [%s]", Message->ID_Equipement[6], Message->ID_Equipement[7], Message->ID_Equipement[8], Message->ID_Equipement[9], Message->ID_Equipement[0], Message->ID_Equipement[1], Message->ID_Equipement[2], Message->ID_Equipement[3], Message->ID_Equipement[4], Message->ID_Equipement[5],Reseau->Nom_Reseau);
	gtk_label_set_text (GTK_LABEL (label_Info_Alert), Texte_Alerte);
	gtk_widget_show (win_Alert);
	g_free (Texte_Alerte);
	Recuperation_Nb_Octet_Recu (sizeof (struct struct_Alerte_Intrus), Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau));

}


void Retour_Notification_Equipement (gpointer * Donnee)
{
	struct struct_Retour_Notification_Changement_Etat_Equipement *Message = (struct struct_Retour_Notification_Changement_Etat_Equipement *) Donnee;
	struct Struct_Modele_Objet *Objet = NULL;
	struct Struct_Liste_Reseau *Reseau = NULL;
	struct Struct_Equipement *Equipement = NULL;
	gchar    *Texte_Notification = NULL;

	ERROR_if (Donnee == NULL);
	Reseau = Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau);
	Objet = Recherche_Objet_Equipement (Message->ID_Equipement, Message->ID_Reseau);
	if (Objet == NULL) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Retour notification (Equipment unknown)");
		return;
	}
	Equipement = (struct Struct_Equipement *) Objet->Pointeur_Attacher;
	Texte_Notification = g_strdup_printf ("Host [%s] is down\nNetwork : [%s]", Equipement->Adresse_Ip_Char,Reseau->Nom_Reseau);
	gtk_label_set_text (GTK_LABEL (label_Info_Alert), Texte_Notification);
	gtk_widget_show (win_Alert);
	g_free (Texte_Notification);
	Texte_Notification = g_strdup_printf ("Host [%s] is down", Equipement->Adresse_Ip_Char);
	Active_Test_Etat_Objet (Objet, Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau));
	Ajouter_Historique (Equipement, Texte_Notification, HISTORIQUE_ALERTE);
	g_free (Texte_Notification);
	Recuperation_Nb_Octet_Recu (sizeof (struct struct_Retour_Notification_Changement_Etat_Equipement), Recupere_Pointeur_Structure_Reseau (Message->ID_Reseau));
}


void Copie_Icone_Equipement (gpointer * pointeur)	
{
	struct Struct_Modele_Objet *Objet = (struct Struct_Modele_Objet *) pointeur;
	struct Struct_Equipement *Equipement = NULL;

	ERROR_if (pointeur == NULL);
	Equipement = (struct Struct_Equipement *) Objet->Pointeur_Attacher;
	if (Equipement->Signature->icone != NULL) {
		Objet->icone = Equipement->Signature->icone;
		Objet->icone_griser = Equipement->Signature->icone_grise;
	}
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Copie Icone Equipement");
}


void Debug_ID_Equipement (unsigned char *ID_Equipement)
{
	ERROR_if (ID_Equipement == NULL);
	printf ("Adresse Mac : %02X:%02X:%02X:%02X:%02X:%02X  IP : %d.%d.%d.%d\n", ID_Equipement[0], ID_Equipement[1], ID_Equipement[2], ID_Equipement[3], ID_Equipement[4], ID_Equipement[5], ID_Equipement[6], ID_Equipement[7], ID_Equipement[8], ID_Equipement[9]);
}


struct Struct_Modele_Objet *Afficher_Nouveau_Equipement (struct Struct_Equipement *Equipement, gint ID_Reseau)
{
	struct Struct_Modele_Objet *Objet_Equipement = NULL;
	gint      Equipment_Type;

	ERROR_if (Equipement == NULL);
	Equipement->Signature = Retour_Pointeur_Signature_Correspondant (Equipement);
	if (Equipement->Signature != NULL) {
		Equipment_Type = Equipement->Signature->Equipment_Type;
		if (Equipment_Type == LISTE_STATION) {	
			if (Cherche_Si_Port_Serveur (Equipement))
				Equipment_Type = LISTE_SERVEUR;
		}
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Equipement : [%s][%s]", Equipement->Signature->System_Name, Equipement->Smb.DomainName);
		if (strlen (Equipement->Smb.DomainName) == 0)
			Objet_Equipement = Ajouter_Nouvelle_Objet (LISTE_MODELE_EQUIPEMENT, Equipement->Adresse_Ip_Char, (gpointer *) Equipement, NULL, Equipment_Type, NULL, PERE_RACINE, ID_Reseau, LISTE_MACHINE);
		else if (strlen (Equipement->Smb.HostName) == 0)
			Objet_Equipement = Ajouter_Nouvelle_Objet (LISTE_MODELE_EQUIPEMENT, Equipement->Adresse_Ip_Char, (gpointer *) Equipement, NULL, LISTE_MODELE_DOMAINE, Equipement->Smb.DomainName, Equipment_Type, ID_Reseau, LISTE_MACHINE);
		else
			Objet_Equipement = Ajouter_Nouvelle_Objet (LISTE_MODELE_EQUIPEMENT, Equipement->Smb.HostName, (gpointer *) Equipement, NULL, LISTE_MODELE_DOMAINE, Equipement->Smb.DomainName, Equipment_Type, ID_Reseau, LISTE_MACHINE);
	}
	else {
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Equipement Inconnu");
		if (Equipement->Nb_Port_Firewaller == Equipement->Nb_Port_Total_Scanner) {
			
			Objet_Equipement = Ajouter_Nouvelle_Objet (LISTE_MODELE_EQUIPEMENT_INCONNU, Equipement->Adresse_Ip_Char, (gpointer *) Equipement, NULL, LISTE_FIREWALL, NULL, PERE_RACINE, ID_Reseau, LISTE_MACHINE);
		}
		else {
			
			Objet_Equipement = Ajouter_Nouvelle_Objet (LISTE_MODELE_EQUIPEMENT_INCONNU, Equipement->Adresse_Ip_Char, (gpointer *) Equipement, NULL, LISTE_INCONNU, NULL, PERE_RACINE, ID_Reseau, LISTE_MACHINE);
		}
	}
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Verifie  Texte_Defaut =[%s]", Objet_Equipement->Texte_Defaut);
	return Objet_Equipement;
}


void Deplace_Equipement (struct Struct_Modele_Objet *Objet, struct Struct_Liste_Reseau *Reseau)
{
	GtkTreeModel *model = NULL;
	guint     Nb_Objet_Liste;

	ERROR_if (Objet == NULL);
	ERROR_if (Reseau == NULL);
	Nb_Objet_Liste = g_list_length (GList_Liste (Objet, Reseau));
	if (Nb_Objet_Liste != 0) {
		if (Objet->Nb_Enfant == 0) {
			
			Recupere_Nouveau_Pointeur_Liste (Objet, Reseau, Supprime_Dans_Liste_Objets (GList_Liste (Objet, Reseau), Objet, TRUE));
			
			model = gtk_tree_view_get_model (GTK_TREE_VIEW (Widget_Liste (Objet, Reseau)));
			gtk_tree_store_remove (GTK_TREE_STORE (model), &Objet->ID_Arbre_Ligne);
			
			if (Objet->Objet_Pere != NULL)
				Supprime_Objet_Liste ((struct Struct_Modele_Objet *) Objet->Objet_Pere, Reseau);
			Afficher_Nouveau_Equipement ((struct Struct_Equipement *) Objet->Pointeur_Attacher, Objet->ID_Reseau);
			
			
			free (Objet);
		}
		else
			debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Deplace objet impossible : objet parent [%s]", Objet->Texte_Defaut);
	}
	else
		debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Deplace objet impossible : aucun objet dans la liste");
}


void Parcours_Equipement_Inconnu (gchar * Signature)
{				
	struct Struct_Liste_Reseau *Reseau = NULL;
	struct Struct_Equipement *Equipement = NULL;
	struct Struct_Modele_Objet *Objet = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Signature == NULL);
	Reseau = Recupere_Pointeur_Structure_Reseau (ID_Reseau_En_Cours);
	Nb_Objet_Liste = g_list_length (Reseau->GList_Liste_Equipement);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Affiche equipements (%d)******************", Nb_Objet_Liste);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Recupere objet suivant %d", i);
		Objet = g_list_nth_data (Reseau->GList_Liste_Equipement, i);
		if (Objet->Pointeur_Attacher != NULL) {
			Equipement = (struct Struct_Equipement *) Objet->Pointeur_Attacher;
			debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%d] : [%s][%s]\n", i, Equipement->Adresse_Ip_Char, Equipement->Adresse_MAC_Char);
			if (strcmp (Signature, Equipement->Md5_Signature) == 0) {
				debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Trouvé");
				Deplace_Equipement (Objet, Reseau);
				Nb_Objet_Liste = g_list_length (Reseau->GList_Liste_Equipement);	
				debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin deplacement");
			}
		}
	}
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Parcours fini");
}

struct Struct_Modele_Objet *Recherche_Objet_Equipement (unsigned char *ID_Equipement, gint ID_Reseau)
{
	struct Struct_Liste_Reseau *Reseau = NULL;
	struct Struct_Equipement *Equipement = NULL;
	struct Struct_Modele_Objet *Objet = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (ID_Equipement == NULL);
	Reseau = Recupere_Pointeur_Structure_Reseau (ID_Reseau);
	Nb_Objet_Liste = g_list_length (Reseau->GList_Liste_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Objet = g_list_nth_data (Reseau->GList_Liste_Equipement, i);
		if (Objet->Pointeur_Attacher != NULL) {
			Equipement = (struct Struct_Equipement *) Objet->Pointeur_Attacher;
			debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%d] : [%s][%s]", i, Equipement->Adresse_Ip_Char, Equipement->Adresse_MAC_Char);
			if (memcmp (Equipement->ID_Equipement, ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1) == 0) {
				debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Trouvé");
				return Objet;
			}
		}
	}
	return NULL;		
}


static unsigned char hexchars[] = "0123456789ABCDEF";

gchar    *url_encode (gchar * source)
{
	gchar    *resultat = NULL;
	int       i, index;


	if (source == NULL)
		return g_strdup ("-null-");
	resultat = (char *) malloc ((sizeof (char)) * ((3 * strlen (source)) + 1));
	ERROR_if (resultat == NULL);
	index = 0;
	for (i = 0; i < strlen (source); i++) {
		resultat[index] = source[i];
		if ((source[i] != '$') && (source[i] != '(') && (source[i] != ')')) {
			if (!(((source[i] <= '9') && (source[i] >= '0'))
					|| ((source[i] <= 'Z') && (source[i] >= 'A'))
					|| ((source[i] <= 'z') && (source[i] >= 'a'))
				)
				) {
				resultat[index] = '%';
				resultat[index + 1] = hexchars[(unsigned char) source[i] >> 4];
				resultat[index + 2] = hexchars[(unsigned char) source[i] & 15];
				index = index + 2;
			}
		}
		index++;
	}
	resultat[index] = '\0';	
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Resultat : [%s]", resultat);
	return resultat;
}


gchar    *Debug_Signature_Equipement (struct Struct_Equipement * Equipement)
{				
	gchar    *ligne = NULL, *debugSignature = NULL;
	gint      tailleDebugSignature;

	ERROR_if (Equipement == NULL);
	
	ligne = g_strdup_printf ("Syn o  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[0].TaillePacketReponse, Equipement->Reponse_Signature[0].HeaderLengthIP, Equipement->Reponse_Signature[0].DSF, Equipement->Reponse_Signature[0].TailleTotal, Equipement->Reponse_Signature[0].Id, Equipement->Reponse_Signature[0].Flags_IP_Offset, Equipement->Reponse_Signature[0].TTL, Equipement->Reponse_Signature[0].NumSeq, Equipement->Reponse_Signature[0].HeaderLengthTCP, Equipement->Reponse_Signature[0].Flags_TCP, Equipement->Reponse_Signature[0].WindowSize);
	tailleDebugSignature = strlen (ligne) + 1;	
	debugSignature = malloc (sizeof (gchar) * tailleDebugSignature);
	ERROR_if (debugSignature == NULL);
	strcpy (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Nul o  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[1].TaillePacketReponse, Equipement->Reponse_Signature[1].HeaderLengthIP, Equipement->Reponse_Signature[1].DSF, Equipement->Reponse_Signature[1].TailleTotal, Equipement->Reponse_Signature[1].Id, Equipement->Reponse_Signature[1].Flags_IP_Offset, Equipement->Reponse_Signature[1].TTL, Equipement->Reponse_Signature[1].NumSeq, Equipement->Reponse_Signature[1].HeaderLengthTCP, Equipement->Reponse_Signature[1].Flags_TCP, Equipement->Reponse_Signature[1].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Ecn o  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[2].TaillePacketReponse, Equipement->Reponse_Signature[2].HeaderLengthIP, Equipement->Reponse_Signature[2].DSF, Equipement->Reponse_Signature[2].TailleTotal, Equipement->Reponse_Signature[2].Id, Equipement->Reponse_Signature[2].Flags_IP_Offset, Equipement->Reponse_Signature[2].TTL, Equipement->Reponse_Signature[2].NumSeq, Equipement->Reponse_Signature[2].HeaderLengthTCP, Equipement->Reponse_Signature[2].Flags_TCP, Equipement->Reponse_Signature[2].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("FUP o  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[3].TaillePacketReponse, Equipement->Reponse_Signature[3].HeaderLengthIP, Equipement->Reponse_Signature[3].DSF, Equipement->Reponse_Signature[3].TailleTotal, Equipement->Reponse_Signature[3].Id, Equipement->Reponse_Signature[3].Flags_IP_Offset, Equipement->Reponse_Signature[3].TTL, Equipement->Reponse_Signature[3].NumSeq, Equipement->Reponse_Signature[3].HeaderLengthTCP, Equipement->Reponse_Signature[3].Flags_TCP, Equipement->Reponse_Signature[3].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Ack o  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[4].TaillePacketReponse, Equipement->Reponse_Signature[4].HeaderLengthIP, Equipement->Reponse_Signature[4].DSF, Equipement->Reponse_Signature[4].TailleTotal, Equipement->Reponse_Signature[4].Id, Equipement->Reponse_Signature[4].Flags_IP_Offset, Equipement->Reponse_Signature[4].TTL, Equipement->Reponse_Signature[4].NumSeq, Equipement->Reponse_Signature[4].HeaderLengthTCP, Equipement->Reponse_Signature[4].Flags_TCP, Equipement->Reponse_Signature[4].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Fin o  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[5].TaillePacketReponse, Equipement->Reponse_Signature[5].HeaderLengthIP, Equipement->Reponse_Signature[5].DSF, Equipement->Reponse_Signature[5].TailleTotal, Equipement->Reponse_Signature[5].Id, Equipement->Reponse_Signature[5].Flags_IP_Offset, Equipement->Reponse_Signature[5].TTL, Equipement->Reponse_Signature[5].NumSeq, Equipement->Reponse_Signature[5].HeaderLengthTCP, Equipement->Reponse_Signature[5].Flags_TCP, Equipement->Reponse_Signature[5].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Syn f  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[6].TaillePacketReponse, Equipement->Reponse_Signature[6].HeaderLengthIP, Equipement->Reponse_Signature[6].DSF, Equipement->Reponse_Signature[6].TailleTotal, Equipement->Reponse_Signature[6].Id, Equipement->Reponse_Signature[6].Flags_IP_Offset, Equipement->Reponse_Signature[6].TTL, Equipement->Reponse_Signature[6].NumSeq, Equipement->Reponse_Signature[6].HeaderLengthTCP, Equipement->Reponse_Signature[6].Flags_TCP, Equipement->Reponse_Signature[6].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Nul f  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[7].TaillePacketReponse, Equipement->Reponse_Signature[7].HeaderLengthIP, Equipement->Reponse_Signature[7].DSF, Equipement->Reponse_Signature[7].TailleTotal, Equipement->Reponse_Signature[7].Id, Equipement->Reponse_Signature[7].Flags_IP_Offset, Equipement->Reponse_Signature[7].TTL, Equipement->Reponse_Signature[7].NumSeq, Equipement->Reponse_Signature[7].HeaderLengthTCP, Equipement->Reponse_Signature[7].Flags_TCP, Equipement->Reponse_Signature[7].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Ecn f  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[8].TaillePacketReponse, Equipement->Reponse_Signature[8].HeaderLengthIP, Equipement->Reponse_Signature[8].DSF, Equipement->Reponse_Signature[8].TailleTotal, Equipement->Reponse_Signature[8].Id, Equipement->Reponse_Signature[8].Flags_IP_Offset, Equipement->Reponse_Signature[8].TTL, Equipement->Reponse_Signature[8].NumSeq, Equipement->Reponse_Signature[8].HeaderLengthTCP, Equipement->Reponse_Signature[8].Flags_TCP, Equipement->Reponse_Signature[8].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("FUP f  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[9].TaillePacketReponse, Equipement->Reponse_Signature[9].HeaderLengthIP, Equipement->Reponse_Signature[9].DSF, Equipement->Reponse_Signature[9].TailleTotal, Equipement->Reponse_Signature[9].Id, Equipement->Reponse_Signature[9].Flags_IP_Offset, Equipement->Reponse_Signature[9].TTL, Equipement->Reponse_Signature[9].NumSeq, Equipement->Reponse_Signature[9].HeaderLengthTCP, Equipement->Reponse_Signature[9].Flags_TCP, Equipement->Reponse_Signature[9].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Ack f  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[10].TaillePacketReponse, Equipement->Reponse_Signature[10].HeaderLengthIP, Equipement->Reponse_Signature[10].DSF, Equipement->Reponse_Signature[10].TailleTotal, Equipement->Reponse_Signature[10].Id, Equipement->Reponse_Signature[10].Flags_IP_Offset, Equipement->Reponse_Signature[10].TTL, Equipement->Reponse_Signature[10].NumSeq, Equipement->Reponse_Signature[10].HeaderLengthTCP, Equipement->Reponse_Signature[10].Flags_TCP, Equipement->Reponse_Signature[10].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Fin f  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[11].TaillePacketReponse, Equipement->Reponse_Signature[11].HeaderLengthIP, Equipement->Reponse_Signature[11].DSF, Equipement->Reponse_Signature[11].TailleTotal, Equipement->Reponse_Signature[11].Id, Equipement->Reponse_Signature[11].Flags_IP_Offset, Equipement->Reponse_Signature[11].TTL, Equipement->Reponse_Signature[11].NumSeq, Equipement->Reponse_Signature[11].HeaderLengthTCP, Equipement->Reponse_Signature[11].Flags_TCP, Equipement->Reponse_Signature[11].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Udp    | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[12].TaillePacketReponse, Equipement->Reponse_Signature[12].HeaderLengthIP, Equipement->Reponse_Signature[12].DSF, Equipement->Reponse_Signature[12].TailleTotal, Equipement->Reponse_Signature[12].Id, Equipement->Reponse_Signature[12].Flags_IP_Offset, Equipement->Reponse_Signature[12].TTL, Equipement->Reponse_Signature[12].NumSeq, Equipement->Reponse_Signature[12].HeaderLengthTCP, Equipement->Reponse_Signature[12].Flags_TCP, Equipement->Reponse_Signature[12].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("IcmpT  | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[13].TaillePacketReponse, Equipement->Reponse_Signature[13].HeaderLengthIP, Equipement->Reponse_Signature[13].DSF, Equipement->Reponse_Signature[13].TailleTotal, Equipement->Reponse_Signature[13].Id, Equipement->Reponse_Signature[13].Flags_IP_Offset, Equipement->Reponse_Signature[13].TTL, Equipement->Reponse_Signature[13].NumSeq, Equipement->Reponse_Signature[13].HeaderLengthTCP, Equipement->Reponse_Signature[13].Flags_TCP, Equipement->Reponse_Signature[13].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	
	ligne = g_strdup_printf ("Ping   | %02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[14].TaillePacketReponse, Equipement->Reponse_Signature[14].HeaderLengthIP, Equipement->Reponse_Signature[14].DSF, Equipement->Reponse_Signature[14].TailleTotal, Equipement->Reponse_Signature[14].Id, Equipement->Reponse_Signature[14].Flags_IP_Offset, Equipement->Reponse_Signature[14].TTL, Equipement->Reponse_Signature[14].NumSeq, Equipement->Reponse_Signature[14].HeaderLengthTCP, Equipement->Reponse_Signature[14].Flags_TCP, Equipement->Reponse_Signature[14].WindowSize);
	tailleDebugSignature = strlen (ligne) + tailleDebugSignature;
	debugSignature = realloc (debugSignature, sizeof (char) * tailleDebugSignature);
	strcat (debugSignature, ligne);
	g_free (ligne);
	return debugSignature;
}


void Afficher_Reponse_Signature (struct Struct_Equipement *Equipement)
{
	gint      i;

	ERROR_if (Equipement == NULL);
	for (i = 0; i < NB_SCAN_OS; i++)
		printf ("%02X %02X %02X %02X %01X %04X %02X %01X %02X %02X %04X\n", Equipement->Reponse_Signature[i].TaillePacketReponse, Equipement->Reponse_Signature[i].HeaderLengthIP, Equipement->Reponse_Signature[i].DSF, Equipement->Reponse_Signature[i].TailleTotal, Equipement->Reponse_Signature[i].Id, Equipement->Reponse_Signature[i].Flags_IP_Offset, Equipement->Reponse_Signature[i].TTL, Equipement->Reponse_Signature[i].NumSeq, Equipement->Reponse_Signature[i].HeaderLengthTCP, Equipement->Reponse_Signature[i].Flags_TCP, Equipement->Reponse_Signature[i].WindowSize);
}


void Recuperation_Nouveau_Equipement (gpointer * Donnee)
{
	struct struct_Nouveau_Equipement *Nouveau = (struct struct_Nouveau_Equipement *) Donnee;
	struct Struct_Equipement *Equipement = NULL;
	struct Struct_Modele_Objet *Objet_Equipement = NULL;
	md5_state_t md_ctx;
	md5_byte_t digest[16];
	gint      i;

	ERROR_if (Donnee == NULL);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Recuperation sur Id_reseau : [%d]\n", Nouveau->ID_Reseau);
	Equipement = (struct Struct_Equipement *) malloc (sizeof (struct Struct_Equipement));
	ERROR_if (Equipement == NULL);
	bzero (Equipement, sizeof (struct Struct_Equipement));
	
	memcpy (Equipement->ID_Equipement, Nouveau->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
	memcpy (Equipement->Adresse_Ip_Char, Nouveau->Adresse_Ip_Char, 15 + 1);
	memcpy (Equipement->Adresse_MAC_Char, Nouveau->Adresse_MAC_Char, 17 + 1);
	Equipement->Equipement_Allumer = TRUE;	
	
	time (&Equipement->Derniere_Modification_Etat_Equipement);
	
	memcpy (Equipement->Reponse_Signature, Nouveau->Reponse_Signature, sizeof (struct struct_Reponse) * NB_SCAN_OS);
	
	memcpy (&Equipement->Smb, &Nouveau->Smb, sizeof (struct struct_Smb));
	
	for (i = 0; i < Nouveau->Nb_Port_Ouvert; i++)
		Ajouter_Nouveau_Port_Equipement (Equipement, Nouveau->Port[i].Num_Port);	
	
	md5_init (&md_ctx);
	md5_append (&md_ctx, (const md5_byte_t *) Equipement->Reponse_Signature, sizeof (struct struct_Reponse) * NB_SCAN_OS);
	md5_finish (&md_ctx, digest);
	sprintf (Equipement->Md5_Signature, "%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X", digest[0], digest[1], digest[2], digest[3], digest[4], digest[5], digest[6], digest[7], digest[8], digest[9], digest[10], digest[11], digest[12], digest[13], digest[14], digest[15]);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Signature : [%s]", Equipement->Md5_Signature);
	Equipement->Debug_Signature = Debug_Signature_Equipement (Equipement);	
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "\nDebug : [%s]", Equipement->Debug_Signature);
	
	if (Nouveau->Intrus)
		Ajouter_Historique (Equipement, "Intrusion Alert", HISTORIQUE_ALERTE);
	else
		Ajouter_Historique (Equipement, "This equipment was found", HISTORIQUE_INFO);
	
	Equipement->Entete_FTP = url_encode (Nouveau->Entete_FTP);
	Equipement->Entete_HTTP = url_encode (Nouveau->Entete_HTTP);
	Equipement->Entete_IMAP = url_encode (Nouveau->Entete_IMAP);
	Equipement->Entete_TELNET = url_encode (Nouveau->Entete_TELNET);
	
	Equipement->Nb_Port_Total_Scanner = Nouveau->Nb_Port_Total_Scanner;
	Equipement->Nb_Port_Ouvert = Nouveau->Nb_Port_Ouvert;
	Equipement->Nb_Port_Fermer = Nouveau->Nb_Port_Fermer;
	Equipement->Nb_Port_Firewaller = Nouveau->Nb_Port_Firewaller;
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nouveau Equipement pour le reseau %d", Nouveau->ID_Reseau);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Hostname : [%s]", Equipement->Smb.HostName);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Ip : %s", Nouveau->Adresse_Ip_Char);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Mac : %s", Nouveau->Adresse_MAC_Char);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete FTP : [%s]", Nouveau->Entete_FTP);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete HTTP : [%s]", Nouveau->Entete_HTTP);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete IMAP : [%s]", Nouveau->Entete_IMAP);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete TELNET : [%s]", Nouveau->Entete_TELNET);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete FTP : [%s]", Equipement->Entete_FTP);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete HTTP : [%s]", Equipement->Entete_HTTP);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete IMAP : [%s]", Equipement->Entete_IMAP);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete TELNET : [%s]", Equipement->Entete_TELNET);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nb port Total scanner : %d", Nouveau->Nb_Port_Total_Scanner);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nb de port ouvert : %d", Nouveau->Nb_Port_Ouvert);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nb de port fermer : %d", Nouveau->Nb_Port_Fermer);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nb de port firewaller : %d", Nouveau->Nb_Port_Firewaller);
	
	Objet_Equipement = Afficher_Nouveau_Equipement (Equipement, Nouveau->ID_Reseau);
	if (Nouveau->Intrus)
		Active_Alerte_Objet (Objet_Equipement, Recupere_Pointeur_Structure_Reseau (Nouveau->ID_Reseau));
	Recuperation_Nb_Octet_Recu (sizeof (struct struct_Nouveau_Equipement), Recupere_Pointeur_Structure_Reseau (Nouveau->ID_Reseau));
}


void Initialisation_Equipement ()
{
	Equipement_Selectionner = NULL;
}
