/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <gnome.h>
#include <stdio.h>
#include "Root_Interface.h"
#include "Combo.h"
#include "../Objet.h"
#include "Settings_Interface.h"
#include "Recherche_Interface.h"
#include "../Image.h"
#include "../Declaration_Constante.h"
#include "Modules.h"
#include "Script_Interface.h"
#include "Add_Network_Interface.h"
#include "Add_Network_Callbacks.h"
#include "../Debug.h"
#include "../Settings_Script.h"
#include "../Settings.h"
#include "../Gconf.h"
#include "../Equipement_Port.h"
#include "../Settings_Port.h"

extern struct Struct_Modele_Objet Liste_Modele_Objet[];


gchar    *Recuperation_Texte_Combo (GtkWidget * combobox)
{
	gchar    *texte = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;

	ERROR_if (combobox == NULL);
	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combobox), &iter)) {
		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
		gtk_tree_model_get (model, &iter, COMBO_TEXT, &texte, -1);
		return texte;
	}
	return NULL;
}


gchar    *Recuperation_Chaine_Attacher_Combo (GtkWidget * combobox)
{
	gchar    *chaine = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;

	ERROR_if (combobox == NULL);
	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combobox), &iter)) {
		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
		gtk_tree_model_get (model, &iter, COMBO_POINTER_CHAINE, &chaine, -1);
		return chaine;
	}
	return NULL;
}

void Efface_Combo (GtkWidget * combobox)
{
	GtkTreeModel *model = NULL;

	
	ERROR_if (combobox == NULL);
	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
	gtk_tree_store_clear (GTK_TREE_STORE (model));
}

void on_combobox_liste_changed (GtkComboBox * combobox, gpointer user_data)
{
	gchar    *texte = NULL, *Chaine = NULL;
	GtkTreeModel *model = NULL;
	void      (*Fonction) ();
	GtkTreeIter iter;

	ERROR_if (combobox == NULL);
	if (gtk_combo_box_get_active_iter (combobox, &iter)) {
		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
		gtk_tree_model_get (model, &iter, COMBO_TEXT, &texte, COMBO_POINTER, &Fonction, COMBO_POINTER_CHAINE, &Chaine, -1);
		if (Fonction != NULL) {	
			(Fonction) ();
		}
		else {
			debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Aucune fonction declarer pour l'item [%s]", texte);
		}
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Chaine attache  [%s]", Chaine);
		g_free (texte);	
	}
}


void Ajoute_Entree_Combo (GtkWidget * combobox, gchar * path_icone, gchar * texte, gboolean sensible, gpointer * Pointeur_Fonction, gchar * Chaine, GtkTreeIter * iter, GtkTreeIter * iter_parent)
{
	GdkPixbuf *pixbuf = NULL;
	GtkTreeModel *model = NULL;

	ERROR_if (combobox == NULL);
	ERROR_if (texte == NULL);
	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
	gtk_tree_store_append (GTK_TREE_STORE (model), iter, iter_parent);
	if (path_icone != NULL) {
		pixbuf = Chargement_Image (path_icone, COULEUR);
		gtk_tree_store_set (GTK_TREE_STORE (model), iter, COMBO_ICON, pixbuf, COMBO_TEXT, texte, COMBO_SENSIBLE, sensible, COMBO_POINTER, Pointeur_Fonction, COMBO_POINTER_CHAINE, Chaine, -1);
	}
	else
		gtk_tree_store_set (GTK_TREE_STORE (model), iter, COMBO_TEXT, texte, COMBO_SENSIBLE, sensible, COMBO_POINTER, Pointeur_Fonction, COMBO_POINTER_CHAINE, Chaine, -1);
}


void creation_combo_milieu (void)
{
	GtkTreeIter iter;

	Ajoute_Entree_Combo (combobox_liste_Modules, PATH_ICONE "Port_Ouvert.png", "Open TCP Ports", TRUE, (gpointer) & Initialisation_Liste_ports, NULL, &iter, NULL);
	if (Recherche_Port_Samba_Ouvert_Sur_Equipement_Selectionner ())
		Ajoute_Entree_Combo (combobox_liste_Modules, PATH_ICONE "Partage_Fichiers.png", "Share", TRUE, (gpointer) & Initialisation_Partage_Fichier_Smb, NULL, &iter, NULL);
	Ajoute_Entree_Combo (combobox_liste_Modules, PATH_ICONE "Nessus.png", "Nessus", TRUE, (gpointer) & Initialisation_Nessus, NULL, &iter, NULL);
	if (Recherche_Port_Telnet_Ouvert_Sur_Equipement_Selectionner ())
		Ajoute_Entree_Combo (combobox_liste_Modules, PATH_ICONE "Telnet.png", "Telnet", TRUE, (gpointer) & Initialisation_Telnet, NULL, &iter, NULL);
	if (Recherche_Port_SSH_Ouvert_Sur_Equipement_Selectionner ())
		Ajoute_Entree_Combo (combobox_liste_Modules, PATH_ICONE "Ssh.png", "SSH", TRUE, (gpointer) & Initialisation_SSH, NULL, &iter, NULL);
	Ajoute_Entree_Combo (combobox_liste_Modules, PATH_ICONE "Port_Ouvert.png", "Script", TRUE, (gpointer) & Initialisation_Script, NULL, &iter, NULL);
	if (Recherche_Port_Samba_Ouvert_Sur_Equipement_Selectionner ())
		Ajoute_Entree_Combo (combobox_liste_Modules, PATH_ICONE "Email.png", "Net Send", TRUE, (gpointer) & Initialisation_NetSend, NULL, &iter, NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_liste_Modules), 0);	
}


void creation_combo_script ()
{
	struct Struct_Liste_Script *Script = NULL;
	gchar    *command = NULL;
	GtkTreeIter iter;
	guint     Nb_Objet_Liste, i;

	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Creation combo script");
	Nb_Objet_Liste = g_list_length (Settings->GList_Script);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Script = g_list_nth_data (Settings->GList_Script, i);
		command = g_strdup_printf ("%s %s", Script->Path, Script->Param);
		if ((Script->Icon != NULL) && (strlen (Script->Icon) != 0))
			Ajoute_Entree_Combo (combobox_Script, Script->Icon, Script->Name, TRUE, NULL, command, &iter, NULL);
		else
			Ajoute_Entree_Combo (combobox_Script, PATH_ICONE "run.png", Script->Name, TRUE, NULL, command, &iter, NULL);
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_Script), 0);	
}


void creation_combo_liste_port ()
{
	struct Struct_Liste_Port *Port = NULL;
	GtkTreeIter iter;
	guint     Nb_Objet_Liste, i;

	Nb_Objet_Liste = g_list_length (Settings->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port = g_list_nth_data (Settings->GList_Port, i);
		if (Port->Port_Pere == NULL)
			Ajoute_Entree_Combo (combobox_Recherche_Port, NULL, Port->Service, TRUE, NULL, NULL, &iter, NULL);
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_Recherche_Port), 0);	
}


void creation_combo_type_port ()
{
	GtkTreeIter iter;

	Ajoute_Entree_Combo (combobox_Liste_Ports_Info_Type, NULL, "Server", TRUE, NULL, NULL, &iter, NULL);
	Ajoute_Entree_Combo (combobox_Liste_Ports_Info_Type, NULL, "Other", TRUE, NULL, NULL, &iter, NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_Liste_Ports_Info_Type), 0);	
}


void creation_combo_type (GtkWidget * Combo)
{
	GtkTreeIter iter;
	int       i;

	ERROR_if (Combo == NULL);
	for (i = 0; i <= NB_TYPE_EQUIPEMENT; i++)
		Ajoute_Entree_Combo (Combo, Liste_Modele_Objet[i].Chemin_Icone, Liste_Modele_Objet[i].Texte_Defaut, TRUE, NULL, NULL, &iter, NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (Combo), 0);	
}


void creation_combo_etat_port (GtkWidget * Combo)
{
	GtkTreeIter iter;

	Ajoute_Entree_Combo (Combo, NULL, "Closed", TRUE, NULL, NULL, &iter, NULL);
	Ajoute_Entree_Combo (Combo, NULL, "Indifferent", TRUE, NULL, NULL, &iter, NULL);
	Ajoute_Entree_Combo (Combo, NULL, "Can be opened", TRUE, NULL, NULL, &iter, NULL);
	Ajoute_Entree_Combo (Combo, NULL, "Must be opened", TRUE, NULL, NULL, &iter, NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (Combo), 0);	
}


void creation_combo_class ()
{
	GtkTreeIter iter;

	Ajoute_Entree_Combo (combobox_Network_Private_Subnet, NULL, "255.0.0.0", TRUE, (gpointer) & Modifie_Class_A, NULL, &iter, NULL);
	Ajoute_Entree_Combo (combobox_Network_Private_Subnet, NULL, "255.255.0.0", TRUE, (gpointer) & Modifie_Class_B, NULL, &iter, NULL);
	Ajoute_Entree_Combo (combobox_Network_Private_Subnet, NULL, "255.255.255.0", TRUE, (gpointer) & Modifie_Class_C, NULL, &iter, NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_Network_Private_Subnet), -1);	
}


void Ajoute_Interface_Combo (gchar * Interface)
{
	GtkTreeIter iter;

	ERROR_if (Interface == NULL);
	Ajoute_Entree_Combo (combobox_Select_Interface, PATH_ICONE "hwinfo.png", Interface, TRUE, NULL, NULL, &iter, NULL);
}


GtkWidget *creation_combo (gboolean Detachable)
{
	GtkCellRenderer *renderer = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeStore *store = NULL;
	GtkWidget *Combo = NULL;

	store = gtk_tree_store_new (COMBO_NUM_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_POINTER, G_TYPE_STRING);
	model = GTK_TREE_MODEL (store);
	Combo = gtk_combo_box_new_with_model (model);
	g_object_unref (model);
	gtk_widget_show (Combo);
	if (Detachable)
		gtk_combo_box_set_add_tearoffs (GTK_COMBO_BOX (Combo), TRUE);	
	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (Combo), renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (Combo), renderer, "pixbuf", COMBO_ICON, "sensitive", COMBO_SENSIBLE, NULL);
	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (Combo), renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (Combo), renderer, "text", COMBO_TEXT, "sensitive", COMBO_SENSIBLE, NULL);
	g_signal_connect ((gpointer) Combo, "changed", G_CALLBACK (on_combobox_liste_changed), NULL);
	return Combo;
}
