/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <gnome.h>
#include "Reseau.h"
#include "Interface/Root_Interface.h"
#include "Interface/Add_Network_Interface.h"
#include "Interface/Bulle_Network_Interface.h"
#include "Com/Initialisation_Com.h"
#include "Declaration_Constante.h"
#include "Objet_Liste.h"
#include "Modules.h"
#include "Debug.h"
#include "Execute.h"
#include "Objet_Bulle.h"
#include "Image.h"

void Affiche_Info_Reseau (struct Struct_Liste_Reseau *Reseau)
{
	gchar    *Text = NULL;
	gdouble   Total, A_Faire;

	if (Reseau == Reseau_Selectionner) {
		Text = g_strdup_printf ("R : %.1f Ko", (((double) Reseau->Octet_Recu) / 1024));
		gtk_label_set_text (GTK_LABEL (label_Utilisation_Reseau_Reception), Text);
		g_free (Text);
		Text = g_strdup_printf ("S : %.1f Ko", (((double) Reseau->Octet_Envoyer) / 1024));
		gtk_label_set_text (GTK_LABEL (label_Utilisation_Reseau_Emission), Text);
		g_free (Text);
		if (Reseau->Nb_Equipement != 0) {
			Total = Reseau->Nb_Equipement;
			A_Faire = Reseau->Nb_Equipement_En_Cours_De_Scan + Reseau->Nb_Equipement_En_Attente_De_Scan;
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), (Total - A_Faire) / Total);
			Text = g_strdup_printf ("Total : %d (%d/%d)   %%%d", Reseau->Nb_Equipement, Reseau->Nb_Equipement_En_Cours_De_Scan, Reseau->Nb_Equipement_En_Attente_De_Scan, (int) (((Total - A_Faire) / Total) * 100));
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), Text);
			g_free (Text);
		}
	}
}



void Arreter_Tous_Les_Agents ()
{
	struct Struct_Liste_Reseau *Reseau = NULL;
	guint     Nb_Objet_Liste, i;

	Nb_Objet_Liste = g_list_length (GList_Liste_Reseau);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Arret des agents (%d)******************", Nb_Objet_Liste);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Reseau = g_list_nth_data (GList_Liste_Reseau, i);
		debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Arret agent %d : [%d][%s]", i, Reseau->Id_Reseau, Reseau->Nom_Reseau);
		Demande_Arret_Agent (Reseau);
	}
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin Arret des agents *****************");
}

void Afficher_Liste_Reseau ()
{
	struct Struct_Liste_Reseau *Reseau = NULL;
	guint     Nb_Objet_Liste, i;

	Nb_Objet_Liste = g_list_length (GList_Liste_Reseau);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Affiche Les reseaux (%d)******************", Nb_Objet_Liste);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Reseau = g_list_nth_data (GList_Liste_Reseau, i);
		debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%d : [%d][%s]", i, Reseau->Id_Reseau, Reseau->Nom_Reseau);
	}
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin Reseau *****************");
}


struct Struct_Liste_Reseau *Recupere_Pointeur_Structure_Reseau (gint ID_Reseau)
{
	struct Struct_Liste_Reseau *Reseau = NULL;
	guint     Nb_Objet_Liste, i;

	
	
	Nb_Objet_Liste = g_list_length (GList_Liste_Reseau);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Reseau = g_list_nth_data (GList_Liste_Reseau, i);
		if (Reseau->Id_Reseau == ID_Reseau)
			return Reseau;
	}
	return NULL;
}

struct Struct_Liste_Reseau *Recupere_Pointeur_Reseau_Selectionner (GtkWidget * scrollwin)
{
	struct Struct_Liste_Reseau *Reseau = NULL;
	guint     Nb_Objet_Liste, i;

	Nb_Objet_Liste = g_list_length (GList_Liste_Reseau);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Reseau = g_list_nth_data (GList_Liste_Reseau, i);
		if (Reseau->scrollwin == scrollwin)
			return Reseau;
	}
	debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Reseau Inconnu");
	return NULL;
}

gboolean Recherche_Si_Reseau_Existe_Deja (gchar * Nom_Reseau)
{
	struct Struct_Liste_Reseau *Reseau = NULL;
	guint     Nb_Objet_Liste, i;

	ERROR_if (Nom_Reseau == NULL);
	Nb_Objet_Liste = g_list_length (GList_Liste_Reseau);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Reseau = g_list_nth_data (GList_Liste_Reseau, i);
		if (strcmp (Reseau->Nom_Reseau, Nom_Reseau) == 0)
			return TRUE;
	}
	return FALSE;
}


struct Struct_Liste_Reseau *Creation_Structure_Nouveau_Reseau (gchar * Nom_Reseau)
{
	struct Struct_Liste_Reseau *Reseau = NULL;

	ERROR_if (Nom_Reseau == NULL);
	
	Reseau = (struct Struct_Liste_Reseau *) malloc (sizeof (struct Struct_Liste_Reseau));
	ERROR_if (Reseau == NULL);
	bzero (Reseau, sizeof (struct Struct_Liste_Reseau));	
	Reseau->Nom_Reseau = g_strdup (Nom_Reseau);
	Reseau->Id_Reseau = Genere_ID_Reseau;
	Reseau->Pid_Agent = Pid_Agent_Initialisation;	
	Genere_ID_Reseau++;	
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Ajouter Nouveau Reseau  Nom[%s]  ID[%d]\n", Reseau->Nom_Reseau, Reseau->Id_Reseau);
	return Reseau;
}


static gboolean expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	struct Struct_Liste_Reseau *Reseau = (struct Struct_Liste_Reseau *) data;

	gdk_draw_drawable (widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], Reseau->pixmap, event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height);
	return TRUE;
}

gboolean Network_leave_notify_event (GtkWidget * widget, GdkEventCrossing * event, gpointer user_data)
{
	gtk_widget_hide (Bulle_Reseau);
	return FALSE;
}


gint Creation_Nouveau_Reseau (gchar * Nom_Reseau)
{
	struct Struct_Liste_Reseau *Reseau = NULL;
	GdkPixbuf *pixbuf = NULL;
	GdkBitmap *mask;
	gchar    *Index_Fin_Nom_Reseau;

	ERROR_if (Nom_Reseau == NULL);
	if (!Recherche_Si_Reseau_Existe_Deja (Nom_Reseau)) {
		Demande_Demarrage_Agent (Genere_ID_Reseau, TRUE);	
		Reseau = Creation_Structure_Nouveau_Reseau (Nom_Reseau);	
		GList_Liste_Reseau = g_list_append (GList_Liste_Reseau, Reseau);	
		
		Reseau->scrollwin = gtk_scrolled_window_new (NULL, NULL);
		gtk_widget_show (Reseau->scrollwin);
		gtk_container_add (GTK_CONTAINER (notebook_reseau), Reseau->scrollwin);
		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (Reseau->scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		
		Reseau->Liste_Equipement = Create_Liste (TRUE, FALSE, TRUE, FALSE);
		gtk_container_add (GTK_CONTAINER (Reseau->scrollwin), Reseau->Liste_Equipement);
		gtk_widget_show (Reseau->Liste_Equipement);
		
		pixbuf = Chargement_Image (gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (iconentry_Network_New)), COULEUR);
		gdk_pixbuf_render_pixmap_and_mask (pixbuf, &Reseau->pixmap, &mask, 127);
		Reseau->drawingarea1 = gtk_drawing_area_new ();
		gtk_widget_ref (Reseau->drawingarea1);
		gtk_widget_shape_combine_mask (Reseau->drawingarea1, mask, 0, 0);
		gtk_widget_set_size_request (Reseau->drawingarea1, 16, 16);
		g_signal_connect (Reseau->drawingarea1, "expose_event", G_CALLBACK (expose_event), (gpointer *) Reseau);
		gtk_widget_set_events (Reseau->drawingarea1, gtk_widget_get_events (Reseau->drawingarea1)
			| GDK_LEAVE_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_LEAVE_NOTIFY);
		g_signal_connect (Reseau->drawingarea1, "motion_notify_event", G_CALLBACK (show_win_bulle_Reseau), (gpointer *) Reseau);
		g_signal_connect ((gpointer) Reseau->drawingarea1, "leave_notify_event", G_CALLBACK (Network_leave_notify_event), (gpointer *) Reseau);
		gtk_widget_show (Reseau->drawingarea1);
		gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_reseau), Reseau->scrollwin, Reseau->drawingarea1);
		
		if (Reseau->Id_Reseau == 0)	
			Reseau_Selectionner = Reseau;
		
		Afficher_Liste_Reseau ();
		
		Index_Fin_Nom_Reseau = index (Nom_Reseau, '-');
		Index_Fin_Nom_Reseau[0] = 0;
		Reseau->Nom = g_strdup (Nom_Reseau);
		Index_Fin_Nom_Reseau[0] = '-';
		Reseau->Interface = g_strdup (Index_Fin_Nom_Reseau + 2);
		
		
		return Reseau->Id_Reseau;
	}
	else {
		debug_printf (INTERFACE, D_INFO_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] : already exist", Nom_Reseau);
		Kill_Process_AutoScan_Agent ();	
		return -1;
	}
	return 0;
}


void Initialisation_Variable_Reseau ()
{
	ID_Reseau_En_Cours = 0;
	GList_Liste_Reseau = NULL;
	Genere_ID_Reseau = 0;
	Reseau_Selectionner = NULL;
}
