/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "Restoration_Agent.h"
#include "Equipement.h"
#include "Equipement_Port.h"
#include "Equipement_Historique.h"
#include "Reseau.h"
#include "Debug.h"
#include "Objet.h"
#include "Com/Commun.h"
#include "Com/Initialisation_Com.h"

void Lit_Donnee_Xml_Restoration_Agent (xmlNodePtr xmlnode)
{
# define FORMAT_DATE "%a %b %e %H:%M:%S %Y"
	gchar    *CharPort = NULL, *Date_Erreur = NULL, *Address_IP = NULL, *Address_MAC = NULL, *Date = NULL;
	struct struct_Restauration_Machine Message;
	gint      Nb_Port_Ajouter, NumPort;
	struct tm tm;

	ERROR_if (xmlnode == NULL);
	bzero (&Message, sizeof (struct struct_Restauration_Machine));
	
	if (!xmlnode->name || g_strcasecmp (xmlnode->name, "Equipment") != 0)
		return;
	Address_IP = xmlGetProp (xmlnode, "Address_IP");
	Address_MAC = xmlGetProp (xmlnode, "Address_MAC");
	strcpy (Message.Adresse_Ip_Char, Address_IP);	
	strcpy (Message.Adresse_MAC_Char, Address_MAC);
	g_free (Address_IP);
	g_free (Address_MAC);
	
	xmlnode = xmlnode->childs;	
	if (xmlnode) {
		while (xmlnode->name && g_strcasecmp (xmlnode->name, "Ports") != 0) {
			
			xmlnode = xmlnode->next;
		}
		if (!xmlnode->name || g_strcasecmp (xmlnode->name, "Ports") != 0)
			return;
		xmlnode = xmlnode->childs;	
		Nb_Port_Ajouter = 0;
		while (xmlnode) {
			if (!xmlnode->name || g_strcasecmp (xmlnode->name, "Port") != 0)
				return;
			CharPort = xmlGetProp (xmlnode, "Num");
			if (sscanf (CharPort, "%d", &NumPort) != 1) {
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration port : [%s]", CharPort);
				NumPort = 0;
			}
			Message.Port[Nb_Port_Ajouter].Num_Port = NumPort;
			g_free (CharPort);
			Date = xmlGetProp (xmlnode, "Last_State_Change");
			Date_Erreur = (gchar *) strptime (Date, FORMAT_DATE, &tm);
			if (Date_Erreur == NULL)
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration date : [%s] [%s]", xmlGetProp (xmlnode, "Last_State_Change"), Date_Erreur);
			Message.Port[Nb_Port_Ajouter].Derniere_Modification_Etat_Port = mktime (&tm);
			g_free (Date);
			Nb_Port_Ajouter++;
			xmlnode = xmlnode->next;	
		}
	}
	Communication_Envoie_Donnee (Pid_Agent_Initialisation, RESTAURATION_MACHINE, (gpointer *) & Message, sizeof (Message), NULL);
}


void Lit_Xml_Restoration_Agent (xmlDocPtr doc)
{
	xmlNodePtr node;

	ERROR_if (doc == NULL);
	
	for (node = doc->root->childs; node != NULL; node = node->next) {
		Lit_Donnee_Xml_Restoration_Agent (node);
	}
}


gboolean Restoration_Agent (char *fname)	
{
	xmlDocPtr doc;

	ERROR_if (fname == NULL);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fichier  : [%s]", fname);
	if (!g_file_exists (fname))	
		debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Error loading file : [%s]", fname);
	
	doc = xmlParseFile (fname);
	if (!doc) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fichier XML restoration : [%s]", fname);
		return FALSE;
	}
	
	if (!doc->root || !doc->root->name || g_strcasecmp (doc->root->name, "AutoScan") != 0) {
		if (!doc->root)
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "l'element n'est pas root");
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fichier XML Port mal formaté : [%s]", fname);
		xmlFreeDoc (doc);
		return FALSE;
	}
	
	Nom_Reseau_Restauration = xmlGetProp (doc->root, "Name");
	if (Nom_Reseau_Restauration == NULL)
		Nom_Reseau_Restauration = "No Name - eth? : xxx.xxx.xxx.xxx";
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nom du reseau restaurer [%s]", Nom_Reseau_Restauration);
	Lit_Xml_Restoration_Agent (doc);
	
	xmlFreeDoc (doc);
	return TRUE;
}
