/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "Restoration_Interface.h"
#include "Equipement.h"
#include "Equipement_Port.h"
#include "Equipement_Historique.h"
#include "Reseau.h"
#include "Debug.h"
#include "Objet.h"
#include "Com/Commun.h"
#include "Com/Initialisation_Com.h"
#include "Reseau.h"
#include "Equipement_Nessus.h"

void Lit_Donnee_Xml_Restoration_Interface (xmlNodePtr xmlnode, gint ID_Reseau)
{
# define FORMAT_DATE "%a %b %e %H:%M:%S %Y"
	struct Struct_Equipement *Equipement = NULL;
	struct Struct_Port *Port_Equipement = NULL;
	struct Struct_Modele_Objet *Objet_Equipement = NULL;
	struct Struct_Historique *Historique = NULL;
	xmlNodePtr xml_Enfant = NULL;
	gchar    *CharPort = NULL, *Date = NULL, *Date_Erreur = NULL, *Address_IP = NULL, *Address_MAC = NULL, *cNotification = NULL;
	gchar    *cSmb = NULL, *cSignature = NULL, *cHistorique = NULL, *CharNiveau = NULL, *cTemp = NULL, *cNessus = NULL;
	struct tm tm;
	gint      NumPort, Erreur_Sscanf, niveau_Historique, NumRegistration, niveau_Nessus;

	ERROR_if (xmlnode == NULL);
	Equipement = (struct Struct_Equipement *) malloc (sizeof (struct Struct_Equipement));
	ERROR_if (Equipement == NULL);
	bzero (Equipement, sizeof (struct Struct_Equipement));
	
	if (!xmlnode->name || g_strcasecmp (xmlnode->name, "Equipment") != 0)
		return;

	Address_IP = xmlGetProp (xmlnode, "Address_IP");
	Address_MAC = xmlGetProp (xmlnode, "Address_MAC");
	if ((Erreur_Sscanf = sscanf (Address_MAC, "%02X:%02X:%02X:%02X:%02X:%02X", (int *) &Equipement->ID_Equipement[0], (int *) &Equipement->ID_Equipement[1], (int *) &Equipement->ID_Equipement[2], (int *) &Equipement->ID_Equipement[3], (int *) &Equipement->ID_Equipement[4], (int *) &Equipement->ID_Equipement[5])) != 6) {
		debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Recuperation Erreur(%d) MAC ID_Equipement %02X:%02X:%02X:%02X:%02X:%02X\n", Erreur_Sscanf);
	}
	if ((Erreur_Sscanf = sscanf (Address_IP, "%d.%d.%d.%d", (int *) &Equipement->ID_Equipement[6], (int *) &Equipement->ID_Equipement[7], (int *) &Equipement->ID_Equipement[8], (int *) &Equipement->ID_Equipement[9])) != 4) {
		debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Recuperation IP ID_Equipement  (%d)", Erreur_Sscanf);
	}
	strcpy (Equipement->Adresse_Ip_Char, Address_IP);
	strcpy (Equipement->Adresse_MAC_Char, Address_MAC);
	g_free (Address_IP);
	g_free (Address_MAC);
	
	Equipement->Equipement_Allumer = TRUE;	
	Date = xmlGetProp (xmlnode, "Last_State_Change");
	Date_Erreur = (gchar *) strptime (Date, FORMAT_DATE, &tm);
	if (Date_Erreur == NULL)
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration Equipement date : [%s] [%s]\n", Date, Date_Erreur);
	Equipement->Derniere_Modification_Etat_Equipement = mktime (&tm);
	g_free (Date);
	Equipement->User_Smb = xmlGetProp (xmlnode, "User_Smb");
	Equipement->Password_Smb = xmlGetProp (xmlnode, "Password_Smb");
	cTemp = xmlGetProp (xmlnode, "Admin_Email");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Contact_Admin = NULL;
	}
	else
		Equipement->Contact_Admin = cTemp;
	cTemp = xmlGetProp (xmlnode, "Localisation");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Localisation = NULL;
	}
	else
		Equipement->Localisation = cTemp;
	cTemp = xmlGetProp (xmlnode, "Serial_Number");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Num_Serie = NULL;
	}
	else
		Equipement->Num_Serie = cTemp;
	cTemp = xmlGetProp (xmlnode, "Serial_Number_Display");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Num_Serie_Ecran = NULL;
	}
	else
		Equipement->Num_Serie_Ecran = cTemp;
	cTemp = xmlGetProp (xmlnode, "Serial_Number_Printer");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Num_Serie_Imprimante = NULL;
	}
	else
		Equipement->Num_Serie_Imprimante = cTemp;
	cTemp = xmlGetProp (xmlnode, "Serial_Number_OS");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Num_Serie_OS = NULL;
	}
	else
		Equipement->Num_Serie_OS = cTemp;
	cTemp = xmlGetProp (xmlnode, "Serial_Number_Office_Suite");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Num_Serie_Office_Suite = NULL;
	}
	else
		Equipement->Num_Serie_Office_Suite = cTemp;
	cTemp = xmlGetProp (xmlnode, "Processor");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Processeur = NULL;
	}
	else
		Equipement->Processeur = cTemp;
	cTemp = xmlGetProp (xmlnode, "Memory");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Memoire = NULL;
	}
	else
		Equipement->Memoire = cTemp;
	cTemp = xmlGetProp (xmlnode, "Disk");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Disque = NULL;
	}
	else
		Equipement->Disque = cTemp;
	cTemp = xmlGetProp (xmlnode, "Contact");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Contact = NULL;
	}
	else
		Equipement->Contact = cTemp;
	cTemp = xmlGetProp (xmlnode, "Tel_Contact");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Tel_Contact = NULL;
	}
	else
		Equipement->Tel_Contact = cTemp;
	cTemp = xmlGetProp (xmlnode, "Notes");
	if ((cTemp != NULL) && (strlen (cTemp) == 0)) {
		g_free (cTemp);
		Equipement->Notes = NULL;
	}
	else
		Equipement->Notes = cTemp;

	cNotification = xmlGetProp (xmlnode, "Notification");
	if (cNotification[0] == '1')
		Equipement->Notification_En_Cours = TRUE;
	else
		Equipement->Notification_En_Cours = FALSE;
	g_free (cNotification);
	xmlnode = xmlnode->childs;
	
	if (xmlnode) {
		
		if (!xmlnode->name || g_strcasecmp (xmlnode->name, "FingerPrinting") != 0)
			return;
		cSignature = xmlGetProp (xmlnode, "MD5");
		strcpy (Equipement->Md5_Signature, cSignature);
		g_free (cSignature);
		Equipement->Entete_FTP = xmlGetProp (xmlnode, "Header_FTP");
		Equipement->Entete_HTTP = xmlGetProp (xmlnode, "Header_HTTP");
		Equipement->Entete_IMAP = xmlGetProp (xmlnode, "Header_IMAP");
		Equipement->Entete_TELNET = xmlGetProp (xmlnode, "Header_TELNET");
		Equipement->Debug_Signature = xmlGetProp (xmlnode, "Debug");
		
		xmlnode = xmlnode->next;
		if (!xmlnode->name || g_strcasecmp (xmlnode->name, "Ports") != 0)
			return;
		cTemp = xmlGetProp (xmlnode, "Open");
		if (sscanf (cTemp, "%d", &Equipement->Nb_Port_Ouvert) != 1) {
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration nb port ouvert : [%s]", cTemp);
			Equipement->Nb_Port_Ouvert = 0;
		}
		g_free (cTemp);
		cTemp = xmlGetProp (xmlnode, "Total");
		if (sscanf (cTemp, "%d", &Equipement->Nb_Port_Total_Scanner) != 1) {
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration nb port total : [%s]", cTemp);
			Equipement->Nb_Port_Total_Scanner = 0;
		}
		g_free (cTemp);
		cTemp = xmlGetProp (xmlnode, "firewalled");
		if (sscanf (cTemp, "%d", &Equipement->Nb_Port_Firewaller) != 1) {
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration nb port firewalled : [%s]", cTemp);
			Equipement->Nb_Port_Firewaller = 0;
		}
		g_free (cTemp);
		xml_Enfant = xmlnode->childs;	
		while (xml_Enfant) {
			if (!xml_Enfant->name || g_strcasecmp (xml_Enfant->name, "Port") != 0)
				return;
			CharPort = xmlGetProp (xml_Enfant, "Num");
			
			if (sscanf (CharPort, "%d", &NumPort) != 1) {
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration port : [%s]", CharPort);
				NumPort = 0;
			}
			g_free (CharPort);
			Port_Equipement = Ajouter_Nouveau_Port_Equipement (Equipement, NumPort);
			Date = xmlGetProp (xml_Enfant, "Last_State_Change");
			Date_Erreur = (gchar *) strptime (Date, FORMAT_DATE, &tm);
			if (Date_Erreur == NULL)
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration Port date : [%s] [%s]", Date, Date_Erreur);
			Port_Equipement->Derniere_Modification_Etat_Port = mktime (&tm);
			g_free (Date);
			cNotification = xmlGetProp (xml_Enfant, "Notification");
			if (cNotification[0] == '1')
				Port_Equipement->Notification_En_Cours = TRUE;
			else
				Port_Equipement->Notification_En_Cours = FALSE;
			g_free (cNotification);
			xml_Enfant = xml_Enfant->next;	
		}
		
		xmlnode = xmlnode->next;
		if (!xmlnode->name || g_strcasecmp (xmlnode->name, "History") != 0)
			return;
		xml_Enfant = xmlnode->childs;	
		while (xml_Enfant) {
			if (!xml_Enfant->name || g_strcasecmp (xml_Enfant->name, "History") != 0)
				return;
			CharNiveau = xmlGetProp (xml_Enfant, "Level");
			if (sscanf (CharNiveau, "%d", &niveau_Historique) != 1) {
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration historique: [%s]", CharNiveau);
				niveau_Historique = 0;
			}
			g_free (CharNiveau);
			cHistorique = xmlGetProp (xml_Enfant, "Text");
			Historique = Ajouter_Historique (Equipement, cHistorique, niveau_Historique);
			g_free (cHistorique);
			Date = xmlGetProp (xml_Enfant, "Date");
			Date_Erreur = (gchar *) strptime (Date, FORMAT_DATE, &tm);
			if (Date_Erreur == NULL)
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration date : [%s] [%s]", Date, Date_Erreur);
			Historique->Date = mktime (&tm);
			g_free (Date);
			xml_Enfant = xml_Enfant->next;	
		}
		
		xmlnode = xmlnode->next;
		if (!xmlnode->name || g_strcasecmp (xmlnode->name, "Smb") != 0)
			return;
		cSmb = xmlGetProp (xmlnode, "Host_Name");
		strcpy (Equipement->Smb.HostName, cSmb);
		g_free (cSmb);
		cSmb = xmlGetProp (xmlnode, "Domain_Name");
		strcpy (Equipement->Smb.DomainName, cSmb);
		g_free (cSmb);
		cSmb = xmlGetProp (xmlnode, "User");
		strcpy (Equipement->Smb.Utilisateur, cSmb);
		g_free (cSmb);
		
		NumRegistration = 0;
		xml_Enfant = xmlnode->childs;	
		while (xml_Enfant) {
			if (!xml_Enfant->name || g_strcasecmp (xml_Enfant->name, "Registration") != 0)
				return;
			cSmb = xmlGetProp (xml_Enfant, "Number");
			if (sscanf (cSmb, "%c", &Equipement->Smb.Smb_Registration[NumRegistration].Numero) != 1) {
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration Registration: [%s]", cSmb);
				Equipement->Smb.Smb_Registration[NumRegistration].Numero = 0;
			}
			g_free (cSmb);
			cSmb = xmlGetProp (xml_Enfant, "Type");
			if (sscanf (cSmb, "%c", &Equipement->Smb.Smb_Registration[NumRegistration].Type) != 1) {
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration Registration: [%s]", cSmb);
				Equipement->Smb.Smb_Registration[NumRegistration].Type = 0;
			}
			g_free (cSmb);
			cSmb = xmlGetProp (xml_Enfant, "Name");
			strcpy (Equipement->Smb.Smb_Registration[NumRegistration].Name, cSmb);
			g_free (cSmb);
			xml_Enfant = xml_Enfant->next;	
			NumRegistration++;
		}
		
		xmlnode = xmlnode->next;
		if (!xmlnode->name || g_strcasecmp (xmlnode->name, "Nessus") != 0)
			return;
		xml_Enfant = xmlnode->childs;	
		while (xml_Enfant) {
			if (!xml_Enfant->name || g_strcasecmp (xml_Enfant->name, "Nessus") != 0)
				break;
			CharNiveau = xmlGetProp (xml_Enfant, "Level");
			if (sscanf (CharNiveau, "%d", &niveau_Nessus) != 1) {
				debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Restoration Nessus: [%s]", CharNiveau);
				niveau_Historique = 0;
			}
			g_free (CharNiveau);
			cNessus = xmlGetProp (xml_Enfant, "Info");
			Ajouter_Info_Nessus (Equipement, cNessus, niveau_Nessus);
			g_free (cNessus);
			cNessus = xmlGetProp (xml_Enfant, "Text");
			Ajouter_Texte_Nessus (Equipement, cNessus);
			g_free (cNessus);
			xml_Enfant = xml_Enfant->next;	
		}
	}
	Objet_Equipement = Afficher_Nouveau_Equipement (Equipement, ID_Reseau);


	
}


void Lit_Xml_Restoration_Interface (xmlDocPtr doc, gint ID_Reseau)
{
	xmlNodePtr node;

	ERROR_if (doc == NULL);
	
	for (node = doc->root->childs; node != NULL; node = node->next) {
		Lit_Donnee_Xml_Restoration_Interface (node, ID_Reseau);
	}
}


gboolean Restoration_Interface (char *fname, gint ID_Reseau)
{
	xmlDocPtr doc;

	ERROR_if (fname == NULL);
	
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fichier  : [%s]", fname);
	if (!g_file_exists (fname))	
		debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Error loading file : [%s]", fname);
	
	doc = xmlParseFile (fname);
	if (!doc) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fichier XML restoration : [%s]", fname);
		return FALSE;
	}
	
	if (!doc->root || !doc->root->name || g_strcasecmp (doc->root->name, "AutoScan") != 0) {
		if (!doc->root)
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "l'element n'est pas root");
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fichier XML Port mal formaté : [%s]", fname);
		xmlFreeDoc (doc);
		return FALSE;
	}
	
	Lit_Xml_Restoration_Interface (doc, ID_Reseau);
	
	xmlFreeDoc (doc);
	return TRUE;
}
