/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/if_ether.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/sockios.h>
#include "Initialisation_Com.h"
#include "Commun.h"
#include "../Debug.h"
#include "../Declaration_Constante.h"
#include "../Notification_Equipement.h"
#include "../Notification_Port.h"
#include "Reseau.h"
#include "Thread_Detect.h"
#include "Thread.h"
#include "../Info.h"
#include "Semaphore.h"

void Envoie_Version_Agent ()
{
	struct struct_Agent_Version Message;
	gint      pid;

	pid = getpid ();
	ERROR_if (strlen (VERSION_AGENT) > TAILLE_MAX_VERSION_AGENT);
	strcpy (Message.Version, VERSION_AGENT);
	Message.pid = pid;
	Communication_Envoie_Donnee (AGENT_VERSION, (gpointer *) & Message, sizeof (Message));
}

void Demarrage_Agent (gpointer * Donnee)
{
	ERROR_if (Donnee == NULL);
	memcpy (&Info_Demarrage, Donnee, sizeof (struct struct_Demarrage_Agent));	
	Debug = Info_Demarrage.Niveau_Debugage;
	Get_My_Ip ();
	Get_My_MAC ();
	Initialisation_Semaphore_Gestion_Thread ();
	demmarrage_Thread (&Boucle_Scan, NULL);
	demmarrage_Thread (&Thread_Detect, NULL);
	demmarrage_Thread (&Thread_Notification_Equipement, NULL);
	demmarrage_Thread (&Thread_Notification_Port, NULL);
	demmarrage_Thread (&Info_Agent, NULL);	
}

void Envoie_Liste_Interface (gpointer * Donnee)
{
	FILE     *Fichier_Interface = NULL;
	struct struct_Nouvelle_Interface Message;
	struct struct_Fin_Liste_Interface Message_Fin;
	struct ifreq ifr;
	struct sockaddr_in sin;
	struct in_addr in;
	gchar     buf[512], *debut, *fin;
	int       Socket;
	gchar     Adresse_Ip[16];

	ERROR_if (Donnee == NULL);
	if ((Fichier_Interface = fopen ("/proc/net/dev", "r")) == NULL)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "/proc/net/dev : %s", strerror (errno));
	fgets (buf, sizeof (buf), Fichier_Interface);	
	fgets (buf, sizeof (buf), Fichier_Interface);
	while (fgets (buf, sizeof (buf), Fichier_Interface)) {
		debut = buf;
		while (debut[0] == ' ')
			debut++;
		fin = index (buf, ':');
		fin[0] = 0;
		debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Interface : %s", debut);
		if (strcmp (debut, "lo") != 0) {	
			
			Socket = socket (AF_INET, SOCK_STREAM, 0);
			if (Socket == -1) {
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
				continue;
			}
			strcpy (ifr.ifr_name, debut);
			if (ioctl (Socket, SIOCGIFADDR, &ifr) != -1) {	
				memcpy (&sin, &ifr.ifr_addr, sizeof (struct sockaddr_in));
				in.s_addr = sin.sin_addr.s_addr;
				strcpy (Adresse_Ip, inet_ntoa (in));	
				strcpy (Message.Interface, debut);
				strcat (Message.Interface, " : ");
				strcat (Message.Interface, Adresse_Ip);
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nouvelle Interface : %s", Message.Interface);
				Communication_Envoie_Donnee (NOUVELLE_INTERFACE, (gpointer *) & Message, sizeof (Message));
			}
			close (Socket);
		}
	}
	Communication_Envoie_Donnee (FIN_LISTE_INTERFACE, (gpointer *) & Message_Fin, sizeof (Message_Fin));
}
