/*
 *  Copyright (C) 2004-2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "Reseau.h"
#include "Debug.h"
#include "Com/Initialisation_Com.h"

void Get_My_MAC ()
{
	struct ifreq ifr;
	struct sockaddr sock;
	int       Socket;

	strcpy (sock.sa_data, Info_Demarrage.Interface);
	sock.sa_family = AF_INET;
	if ((Socket = socket_control (AF_INET, SOCK_PACKET, htons (ETH_P_ALL))) < 0) {
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
	ifr.ifr_addr.sa_family = AF_INET;
	sprintf (ifr.ifr_name, Info_Demarrage.Interface);
	if (ioctl (Socket, SIOCGIFHWADDR, &ifr) == -1) {
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
	memcpy (Mon_Adresse_MAC, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	if (close_control (Socket) < 0) {
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
}


void Get_My_Ip ()
{
	struct ifreq ifr;
	struct sockaddr_in sin;
	struct sockaddr sock;
	int       Socket;

	strcpy (sock.sa_data, Info_Demarrage.Interface);
	sock.sa_family = AF_INET;
	if ((Socket = socket_control (AF_INET, SOCK_PACKET, htons (ETH_P_ALL))) < 0) {
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
	strcpy (ifr.ifr_name, Info_Demarrage.Interface);
	if (ioctl (Socket, SIOCGIFADDR, &ifr) == -1) {
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
	memcpy (&sin, &ifr.ifr_addr, sizeof (struct sockaddr_in));
	memcpy (Mon_Adresse_Ip, &sin.sin_addr.s_addr, IP_ADDR_LEN);
	if (close_control (Socket) < 0) {
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
}


unsigned short ckecksum (char *donnee, int longueur, int valeur)
{
	int       i;
	int       somme;
	int       Nombre;

	i = 0;
	somme = 0;
	while (i < longueur) {
		if (longueur == (i + 1)) {
			donnee[i + 1] = 0;
		}
		Nombre = (((donnee[i] << 8) & 0xFF00) + (donnee[i + 1] & 0xFF));
		somme = somme + Nombre;
		i = i + 2;
	}
	while ((somme >> 16) != 0) {
		somme = (somme >> 16) + (somme & 0xffff);
	}
	somme = (0xFFFF - (somme & 0xFFFF)) - valeur;
	if ((valeur != 0) && (somme < 0))
		somme--;
	return (unsigned short) somme;
}


int waiting_service (int sock, int event)
{
	struct pollfd fds;
	int       resultat;

	bzero (&fds, sizeof fds);	
	fds.events = event;
	fds.fd = sock;		
	if ((resultat = poll (&fds, 1, 1000)) > 0) {	
		if ((fds.revents & POLLERR) == POLLERR) {
			return ERREUR_SOCK;
		}
		if ((fds.revents & POLLHUP) == POLLHUP) {
			return ERREUR_SOCK;
		}
		return DONNEE_PRET;
	}
	else {
		if (resultat != 0) {
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		}
		return ATTENDRE;
	}
}


gboolean attendre_Donnee (int sock, int temp)
{
	int       result;

	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Attendre_donnee");
	while ((result = waiting_service (sock, POLLIN)) == ATTENDRE) {
		debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, ".(%d)", temp);
		temp--;
		if (temp == 0) {
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Timeout dans Attendre_donnee");
			return FALSE;
		}
	}
	if (result == ERREUR_SOCK)
		return FALSE;
	return TRUE;
}


gboolean attendre_Sock (int sock)
{
	int       temp;
	int       result;

	temp = 5;
	while ((result = waiting_service (sock, POLLOUT)) == ATTENDRE) {
		debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, ".(%d)", temp);
		temp--;
		if (temp == 0) {
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Timeout dans Attendre_donnee");
			return FALSE;
		}
	}
	if (result == ERREUR_SOCK)
		return FALSE;
	return TRUE;
}


void set_NonBlock (int sock)
{
	int       flags;

	if ((flags = fcntl (sock, F_GETFL)) == -1)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	if (fcntl (sock, F_SETFL, flags | O_NONBLOCK) == -1)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
}


gboolean connection_Serveur (char *serveur, struct hostent *host, int Port, int sock)	
{
	struct sockaddr_in SocketAddr;


	memset (&SocketAddr, 0, sizeof (struct sockaddr_in));
	memcpy (&SocketAddr.sin_addr.s_addr, host->h_addr, host->h_length);
	SocketAddr.sin_family = AF_INET;
	SocketAddr.sin_port = htons (Port);
	if (connect (sock, (struct sockaddr *) &SocketAddr, sizeof (struct sockaddr)) < 0) {
		if (errno != EINPROGRESS) {
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
			if (close_control (sock) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
			return FALSE;
		}
		if (attendre_Sock (sock) == FALSE) {
			if (close_control (sock) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
			return FALSE;
		}
	}
	return TRUE;		
}


struct hostent *resoudre_DNS (char *Serveur)
{
	struct hostent *host;

	host = gethostbyname (Serveur);	
	if (host == NULL) {
		switch (h_errno) {
		    case HOST_NOT_FOUND:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "L'hôte indiqué est inconnu. [%s]", Serveur);
			    break;
		    case NO_ADDRESS:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Le nom est valide mais ne possède pas d'adresse IP. [%s]", Serveur);
			    break;
		    case NO_RECOVERY:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Une erreur fatale du serveur de noms est apparue. [%s]", Serveur);
			    break;
		    case TRY_AGAIN:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Aucune connection Ã  internet. [%s]", Serveur);
			    break;
		    default:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur Inconnue [%s]", Serveur);
			    break;
		}
	}
	return host;
}


int Creation_Socket_TCP ()
{
	int       sock;

	sock = socket_control (AF_INET, SOCK_STREAM, 0);	
	if (sock == -1) {
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		return FALSE;
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Creation d'une socket TCP/IP");
	return sock;
}


int ConnectionServeur (char *Serveur, int Port)
{
	int       IdSocket, code_erreur;
	struct hostent *host;
	struct sockaddr_in SocketAddr;

	ERROR_if (Serveur == NULL);
	IdSocket = Creation_Socket_TCP ();
	set_NonBlock (IdSocket);
	if ((host = resoudre_DNS (Serveur)) == NULL) {	
		if (close_control (IdSocket) < 0)
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		return (-1);
	}
	memset (&SocketAddr, 0, sizeof (struct sockaddr_in));
	memcpy (&SocketAddr.sin_addr.s_addr, host->h_addr, host->h_length);
	SocketAddr.sin_family = AF_INET;
	SocketAddr.sin_port = htons (Port);
	if ((code_erreur = connect (IdSocket, (struct sockaddr *) &SocketAddr, sizeof (struct sockaddr))) < 0) {
		if (errno != EINPROGRESS) {
			debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
			if (close_control (IdSocket) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
			return -1;
		}
		if (attendre_Sock (IdSocket) == FALSE) {
			if (close_control (IdSocket) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
			return -1;
		}
	}
	return IdSocket;
}
